from json import JSONDecodeError
import logging

from fastapi import WebSocket, APIRouter, WebSocketDisconnect

from gisaf.live import live_server

logger = logging.getLogger(__name__)

api = APIRouter(
    tags=["websockets"],
    responses={404: {"description": "Not found"}},
)


@api.websocket("/live/{store}")
async def live_layer(store: str, websocket: WebSocket):
    """
    Websocket for live layer updates
    """
    await websocket.accept()
    try:
        while True:
            try:
                msg_data = await websocket.receive_json()
            except JSONDecodeError:
                msg_text = await websocket.receive_text()
                if msg_text == "close":
                    await websocket.close()
                continue
            # else:
            if "message" in msg_data:
                if msg_data["message"] == "subscribeLiveLayer":
                    live_server.add_subscription(websocket, store)
                elif msg_data["message"] == "unsubscribeLiveLayer":
                    live_server.remove_subscription(websocket, store)
            else:
                logger.warning(
                    f"Got websocket message with no message field: {msg_data}"
                )
    except WebSocketDisconnect:
        logger.debug("Websocket disconnected")
        # logger.debug('websocket connection closed')
